package com.andreani.xtol.node;

import com.andreani.xtol.util.*;
import net.jini.core.entry.*;
import net.jini.lookup.entry.*;
import net.jini.core.lookup.ServiceID;
/**
 * This class represents a server and is used to register an available server
 * into the lookup service.
 * This compiled class must be accesible from the lookup service in order to
 * use the browser utility from Sun's Jini reference implementation. Simply
 * put the class into reggie-dl.jar for Jini 1.1
 *
 * @author    Stefano Andreani
 * @version   1.0
 */
public class ServerNode implements java.io.Serializable
{
   /**
    * IP address of the server
    */
   public String serverIP;
   /**
    * Port number associated to the service
    */
   public Integer serverPort;
   /**
    * Actual load of this server. It could be useful to realize intelligent
    * load balancing policies.
    */
   public Double serverLoad;
   final static String SERVICE_NAME = "Jini Service - Server Node";
   final static String SERVICE_MFR = "Stefano Andreani";
   final static String SERVICE_VENDOR = "DEIS";
   final static String SERVICE_VERSION = "1.0";
   final static String SERVICE_COMMENT = "This entity represent a server node.";
   final static String LOCATION_FLOOR = "second";
   final static String LOCATION_ROOM = "lab2";
   final static String LOCATION_BUILDING = "DEIS building";
   final static String CLASS_NAME = "serverNode";


   /**
    * Constructor for the ServerNode object
    *
    * @param ip    IP address
    * @param port  Port number
    * @param load  Actual load of the server
    */
   public ServerNode(String ip, int port, double load)
   {
      serverIP = ip;
      serverPort = new Integer(port);
      serverLoad = new Double(load);
   }


   /**
    * Gets the Entries attribute of the ServerNode object. This method returns
    * standard fields used by Jini services to lookup a service.
    *
    * @return   The Entries value
    */
   public Entry[] getEntries()
   {
      Entry myEntry[] = new Entry[4];
      myEntry[0] = new ServiceInfo(SERVICE_NAME, SERVICE_MFR, SERVICE_VENDOR, SERVICE_VERSION, "", "");
      myEntry[1] = new Name(CLASS_NAME);
      myEntry[2] = new Comment(SERVICE_COMMENT);
      myEntry[3] = new Location(LOCATION_FLOOR, LOCATION_ROOM, LOCATION_BUILDING);
      return myEntry;
   }
}
