//Source file: C:\\1dati\\universita\\reti\\progetto\\extremeTolerance\\src\\com\\andreani\\xtol\\BindingEventHandler.java

package com.andreani.xtol.node;

import com.andreani.xtol.util.*;
import javax.servlet.ServletContext;
import javax.servlet.http.*;
import net.jini.core.lookup.ServiceRegistrar;
import com.andreani.util.HttpFunc;
import java.util.Hashtable;
import java.net.InetAddress;
import org.apache.log4j.Category;
import org.apache.log4j.BasicConfigurator;

/**
 * This class must be instantiated by the servlet container. To do so you can
 * include the following code in the deployment descriptor:
 * <pre>
 * &lt;web-app&gt;
 * &lt;display-name&gt;MyApplication&lt;/display-name&gt;
 * &lt;listener&gt;
 * &lt;listener-class&gt;com.andreani.xtol.node.SessionEventHandler&lt;/listener-class&gt;
 * &lt;/listener&gt;
 * &lt;servlet&gt;
 * &lt;display-name&gt;MyServlet&lt;/display-name&gt;
 * ...etc
 * &lt;/servlet&gt;
 * &lt;/web-app&gt;
 * </pre>
 *
 * The container is required to complete instantiation of the listener
 * classes in a web application prior to the start of execution of the first
 * request into the application. The container must reference each listener
 * instance until the last request is serviced for the web application.
 *
 * @author    Stefano Andreani
 * @version   1.0
 */
public class SessionEventHandler implements HttpSessionListener,
      HttpSessionAttributeListener
{
   /**
    * Log handler (Log4Java)
    */
   protected Category cat;
   SessionSpaceManager spacem;


   /**
    * Constructor for the SessionEventHandler object
    */
   public SessionEventHandler()
   {
      cat = Category.getInstance(this.getClass().getName());
      BasicConfigurator.configure();
      cat.debug("constructor");
      spacem = new SessionSpaceManager();
   }


   /**
    * The constructor searches SessionSpace services through the lookup service
    * and registers itself for notifications of new SessionSpace services.
    *
    * @param registrar  Description of Parameter
    * @roseuid          3AF672FB03C3
    */
   public SessionEventHandler(ServiceRegistrar registrar)
   {
   }


   /**
    * Description of the Method
    *
    * @param se  Description of Parameter
    */
   public void sessionCreated(HttpSessionEvent se)
   {
      cat.debug("Local Session Object created!");
   }


   public void sessionDestroyed(HttpSessionEvent se)
   {
      cat.info("Session Object destroyed! I must handle this case.");
   }


   /**
    * When an attribute is added to a session this method propagates
    * modifications to the SessionSpaces
    *
    */
   public void attributeAdded(HttpSessionBindingEvent se)
   {
      cat.debug("attributeAdded");
      HttpSession session = se.getSession();
      Hashtable hashSession = HttpFunc.session2hash(session);
      String globalSessionId = (String)
            session.getAttribute(Global.COOKIE_NAME_SESSIONID);
      try
      {
         ServletContext sctx = session.getServletContext();
         String port = sctx.getInitParameter("port");
         cat.debug("port="+port);
         SessionEntry entry = new SessionEntry(globalSessionId,
               InetAddress.getLocalHost().getHostAddress()+":"+port,
               hashSession, new Boolean(false));
         cat.debug("entry="+entry);
         spacem.updateSession(entry);
      }
      catch (Exception ex)
      {
         cat.error("Error: " + ex.getMessage(), ex);
      }

   }


   /**
    * When an attribute is removed from a session this method propagates
    * modifications to the SessionSpaces
    *
    */
   public void attributeRemoved(HttpSessionBindingEvent se)
   {
      attributeAdded(se);
   }


   /**
    * When an attribute is replaced into a session this method propagates
    * modifications to the SessionSpaces
    *
    */
   public void attributeReplaced(HttpSessionBindingEvent se)
   {
      attributeAdded(se);
   }
}
