//
// -- Java Code Generation Process --

package com.andreani.xtol.proxy;

// Import Statements
import com.andreani.xtol.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import java.net.*;
import net.jini.core.lookup.ServiceRegistrar;
import net.jini.discovery.*;
import org.apache.log4j.Category;
import org.apache.log4j.BasicConfigurator;
import java.io.*;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.StringTokenizer;
import java.util.NoSuchElementException;
import net.jini.core.entry.*;
import net.jini.lookup.entry.*;
import net.jini.entry.*;
import net.jini.discovery.*;
import net.jini.core.lookup.*;
import java.rmi.*;

/**
 * This is
 *
 * @author    Stefano Andreani
 * @version   1.0
 */
public class ForwServlet extends HttpServlet
{
   /**
    * Log handler (Log4Java)
    */
   protected Category cat;
   /**
    * This local cache contains the mapping between global session id
    * and destination server address of each client.
    */
   protected Hashtable serverAddr;
   //  Hashtable is synchronized: no concurrency problems (must use
   // synchronized(serverAddr) {...})

   SessionSpaceManager spacem;
   ServerManager serverm;


   /**
    * Constructor for the ForwServlet object
    */
   public ForwServlet()
   {
      super();
      cat = Category.getInstance(this.getClass().getName());
      BasicConfigurator.configure();
      cat.debug("contructor");

   }

   /**
    * Servlet initializzation
    *
    * @exception ServletException  Exception thrown by init
    */
   public void init() throws ServletException
   {
      cat.debug("ForwServlet inizialization");
      serverm = new ServerManager();
      serverAddr = new Hashtable();
      spacem = new SessionSpaceManager();
   }

   public void destroy()
   {
      serverm.terminate();
      spacem.terminate();
   }


   public void doPost(HttpServletRequest req, HttpServletResponse resp)
          throws ServletException, IOException
   {
      doGet(req, resp);
   }


   public void doGet(HttpServletRequest req, HttpServletResponse resp)
          throws ServletException, IOException
   {
      cat.debug("doGet()");
      spacem.snoopSpace();
      try
      {
         proxy(req, resp);
         //reqHeaders(req,resp);
      }
      catch (Exception ex)
      {
         cat.error(ex.getMessage(),ex);
      }
      spacem.snoopSpace();
   }

   void reqHeaders(HttpServletRequest request, HttpServletResponse response)
   throws IOException
   {

        response.setContentType("text/html");
        PrintWriter out = response.getWriter();
        Enumeration e = request.getHeaderNames();
        while (e.hasMoreElements()) {
            String name = (String)e.nextElement();
            String value = request.getHeader(name);
            out.println(name + " = " + value);
         }
   }

   /**
    * Simulate a proxy passing the client request to the right server and
    * returning the response from the server to the client. If the server fails
    * this method try to call another server. The actual implementation has some
    * bugs because it isn't transparent, i.e. it doesn't show client properties
    * (or cookies) to the server. I can improve it!
    *
    * @param resp           proxy response
    * @param req            client request
    * @exception Exception
    */
   void proxy(HttpServletRequest req, HttpServletResponse resp) throws Exception
   {
      cat.debug("proxy()");
      Cookie c = retrieveCookie(req, resp);
      cat.debug("retrieveCookie(req, resp)="+c);
      boolean oldCookie = c!=null && (serverAddr.get(c.getValue()))==null;
      if (c == null || oldCookie)
      {
         cat.debug("c == "+c+" oldCookie="+oldCookie);
         c = createGuid(req, resp);
         cat.debug("createGuid(req, resp)="+c);
      }
      String guid = c.getValue();
      debugSpace(guid);
      String queryString = req.getQueryString();
      String path = req.getRequestURI();

      // Passes to the server node the guid cookie, all client cookies and,
      // if required, the fail cookie.
      StringBuffer clientCookies = new StringBuffer();
      cat.debug("clientCookies.append(");
      clientCookies.append(Global.COOKIE_NAME_SESSIONID + "=" + guid);
      cat.debug("req.getCookies()");
      Cookie[] cookies = req.getCookies();
      if (cookies!=null) for (int i=0; i<cookies.length; i++)
      {
         cat.debug("for (int i=0; i<cookies.length; i++) cookies.length="+cookies.length);
         // A guid cookie is already present if this isn't the first
         // client request. If there is one I don't copy it again:
         if (cookies[i].getName().equals(Global.COOKIE_NAME_SESSIONID)) {}
         else
         {
            clientCookies.append("; "+cookies[i].getName()+"="+cookies[i].getValue());
         }
      }
      cat.debug("clientCookies="+clientCookies);
      // failover...
      boolean done = false;
      // Number of faults:
      int failure = 0;
      HttpURLConnection con = null;
      while (!done)
      {
         URL url;
         String destServer = (String) serverAddr.get(guid);
         if (queryString == null)
            url = new URL("http://"+destServer+path);
         else url = new URL("http://"+destServer+path+"?"+queryString);
         cat.debug("url.openConnection();");
         con = (HttpURLConnection) url.openConnection();
         // Set the client headers to send to the server node:
         for (Enumeration headerNames = req.getHeaderNames();
               headerNames.hasMoreElements(); )
         {
            String hName = (String) headerNames.nextElement();
            /*cat.debug("con.setRequestProperty(hName, req.getHeader(hName)); hname="+
               hName+" req.getHeader(hName)="+req.getHeader(hName));*/
            con.setRequestProperty(hName, req.getHeader(hName));
         }
         if (failure == 1)
         {
            clientCookies.append("; "+Global.COOKIE_FAIL_OVER + "=true");
         }
         cat.debug("con.setRequestProperty(");
         // overwrites an eventual cookie property:
         con.setRequestProperty("Cookie", clientCookies.toString());
         try
         {
            cat.debug("con.connect();");
            con.connect();
            done = true;
            cat.debug("done:con="+con.getURL());
         }
         catch (IOException ex)
         {
            // Single failure is the most frequent
            failure++;
            cat.info("Connection to "+destServer+" failed: looking for an available server.");
            serverm.serviceFailure(destServer);
            String newDestServer = serverm.findNextServer();
            // For now I assign all the sessions to the same server:
            // a round robin policy could be better.
            spacem.snoopSpace();
            spacem.updateReferences(destServer, newDestServer);
            spacem.snoopSpace();
            synchronized(serverAddr)
            {
               for (Enumeration e = serverAddr.elements(); e.hasMoreElements() ;)
               {
                  String currentAddr = (String) e.nextElement();
                  if (currentAddr.equals(destServer))
                  {
                     serverAddr.remove(guid);
                     serverAddr.put(guid,newDestServer);
                  }
               }
            }
         }
      }
      spacem.snoopSpace();
      /* When a failure occurs the following operations are needed:
         (the client isn't affected by the failure because its guid remains valid)
         - find an available space
         - modify related SessionEntries choosing a new destination server and
           update serverAddr mapping table
           (pay attenction to assign to the current user session the same server
           choose now)
         - try to connect to the new server
         retrieveCookie in the new destination node must wait for the new
         session to be available (because operations are asyncrone may be
         that the JavaSpace is not updated when the node asks for the object)
      */
      // Reads all the headers passed from the server node to the proxy and
      // copy them to the client:
      for (int i=1; ;i++ )
      {
         String hkey = con.getHeaderFieldKey(i);
         if (hkey == null)
         {
            break;
         }
         if(hkey.equals("Set-Cookie"))
         {
            // Typical Set-Cookie header (Tomcat 4 beta 5):
            // Set-Cookie: JSESSIONID=E248549582C7F1B20269F092D3C40350;Path=/xtol-node
            // http://www.netscape.com/newsref/std/cookie_spec.html
            String hvalue = con.getHeaderField(i);
            cat.debug(hkey+": "+hvalue);
            Cookie cookie = null;
            StringTokenizer st = new StringTokenizer(hvalue,";");
            try
            {
               // The first token is alwais the name=value pair:
               String token = st.nextToken().trim();
               int valueIndex = token.indexOf("=")+1;
               String name = token.substring(0,valueIndex-1);
               String value = token.substring(valueIndex);
               cookie = new Cookie(name,value);
            }
            catch (NoSuchElementException ex)
            {
               cat.error(ex.getMessage(),ex);
            }
            while (st.hasMoreTokens())
            {
               String token = st.nextToken().trim();
               int valueIndex = token.indexOf("=")+1;
               if (token.startsWith("Path="))
                  cookie.setPath(token.substring(valueIndex));
                  //cookie.setPath("/");
               if (token.startsWith("Domain="))
                  cookie.setDomain(token.substring(valueIndex));
               if (token.startsWith("Comment="))
                  cookie.setComment(token.substring(valueIndex));
               if (token.startsWith("Max-age="))
                  cookie.setMaxAge(Integer.valueOf(token.substring(valueIndex)).intValue());
               if (token.startsWith("Version="))
                  cookie.setVersion(Integer.valueOf(token.substring(valueIndex)).intValue());
               if (token.startsWith("secure"))
                  cookie.setSecure(true);
            }
            resp.addCookie(cookie);
            cat.debug("added cookie "+cookie.getName()+"="+cookie.getValue());
         }
      }
      BufferedReader in = new BufferedReader(new InputStreamReader(
            con.getInputStream()));
      cat.debug("resp.getWriter()");
      PrintWriter out = resp.getWriter();
      for (; ; )
      {
         //cat.debug("in.readLine()");
         String line = in.readLine();
         //cat.debug("in.readLine()="+line);
         if (line == null)
         {
            break;
         }
         out.println(line);
      }
      con.disconnect();
      cat.debug("con.disconnect();");
   }


   /**
    * Handles a failure removing references to failed server
    *
    * @param failedGuid
    * @param req
    * @param res
    * @return
    */
   String faultHandler(String failedGuid, HttpServletRequest req, HttpServletResponse res)
   {
      String newGuid=null;
      serverm.serviceFailure(failedGuid);
      return newGuid;
   }

   void debugSpace(String sessionid)
   {
      cat.debug("debugSpace()");
      SessionEntry se = spacem.retrieveEntry(sessionid);
      cat.debug(se);
   }

   /**
    * Choose the destination server querying the ServerManager and add it to
    * the serverAddr hashtable, and create the SessionEntry through the
    * SessionSpaceManager.
    *
    * @param req
    * @param resp
    * @return
    */
   Cookie createGuid(HttpServletRequest req, HttpServletResponse resp)
   {
      cat.debug("createGuid()");
      String locIPAddr = null;
      try
      {
         locIPAddr = InetAddress.getLocalHost().getHostAddress();
      }
      catch (java.net.UnknownHostException ex)
      {
         cat.error(ex.getMessage(),ex);
      }
      java.rmi.server.UID uid = new java.rmi.server.UID();
      String guid = uid.toString()+":"+locIPAddr;
      cat.debug("guid="+guid);
      Cookie c = new Cookie(Global.COOKIE_NAME_SESSIONID, guid);
      cat.debug("resp.addCookie(c)");
      resp.addCookie(c);
      cat.debug("serverm.findNextServer()");
      String nextServer = serverm.findNextServer();
      cat.debug("nextserver="+nextServer);
      synchronized(serverAddr)
      {
         serverAddr.put(guid, nextServer);
      }
      cat.debug("spacem.insertEmptyEntry(guid, nextServer)");
      spacem.insertEmptyEntry(guid, nextServer);
      return c;
   }


   /**
    * If the cookie doesn't exist invokes the load balancer to assign a server
    * and returns its address.
    *
    * @param req
    * @param resp
    * @return
    */
   Cookie retrieveCookie(HttpServletRequest req, HttpServletResponse resp)
   {
      cat.debug("retrieveCookie()");
      Cookie[] cv = req.getCookies();
      if (cv == null) return null;
      cat.debug("Cookie[]="+cv+" cv.length="+cv.length);
      for (int i = 0; i < cv.length; i++)
      {
         cat.debug("Cookie["+i+"]="+cv[i].getName());
         if (cv[i].getName().equals(Global.COOKIE_NAME_SESSIONID))
         {
            return cv[i];
         }
      }
      return null;
   }
}
