//Source file: C:\\1dati\\universita\\reti\\progetto\\extremeTolerance\\src\\com\\andreani\\xtol\\ServerManager.java

package com.andreani.xtol.proxy;

import com.andreani.xtol.util.*;
import com.andreani.xtol.node.*;
import net.jini.lookup.ServiceDiscoveryEvent;
import net.jini.lookup.ServiceDiscoveryListener;
import net.jini.lookup.ServiceDiscoveryManager;
import net.jini.lookup.LookupCache;
import net.jini.core.entry.Entry;
import net.jini.discovery.LookupDiscovery;
import net.jini.discovery.LookupDiscoveryManager;
import net.jini.core.lookup.ServiceTemplate;
import net.jini.core.lookup.ServiceID;
import net.jini.core.lookup.ServiceItem;
import java.rmi.RMISecurityManager;
import org.apache.log4j.Category;
import org.apache.log4j.BasicConfigurator;
import java.util.*;

/**
 * This class manages the list of available servers. This class is thread safe.
 * @author    Stefano Andreani
 * @version   1.0
 */
public class ServerManager implements ServiceDiscoveryListener
{
   /**
    * Log handler (Log4Java)
    */
   protected Category cat;
   Vector serverlist;
   Hashtable serverv;
   Hashtable addrMap;
   int lastChoice;
   private ServiceDiscoveryManager clientMgr = null;


   /**
    * Constructor for the ServerManager object
    */
   public ServerManager()
   {
      cat = Category.getInstance(this.getClass().getName());
      //BasicConfigurator.configure();

      cat.debug("constructor");
      serverv = new Hashtable(10);
      addrMap = new Hashtable(10);
      serverlist = new Vector(10);
      if (System.getSecurityManager() == null)
      {
         System.setSecurityManager(new RMISecurityManager());
      }
      try
      {
         LookupDiscoveryManager mgr = new LookupDiscoveryManager(
               LookupDiscovery.ALL_GROUPS, null, null);
         clientMgr = new ServiceDiscoveryManager(mgr, null);
         Class[] classes = new Class[]{ServerNode.class};
         ServiceTemplate template = new ServiceTemplate(null, classes, null);
         LookupCache cache = clientMgr.createLookupCache(template, null, this);
      }
      catch (Exception ex)
      {
         cat.error(ex.getMessage(), ex);
      }
   }


   /**
    * This method returns the address of an available server. It follows
    * a round robin rule.
    * This is the method to override if you want to change manager policy.
    *
    * @return   address of the next server to invoke
    */
   public synchronized String findNextServer()
   {
      cat.debug("findNextServer");
      lastChoice++;
      cat.debug("lastChoice=" + lastChoice + " serverv.size()=" + serverlist.size());
      if (lastChoice >= serverlist.size())
      {
         lastChoice = 0;
      }
      ServerNode server = (ServerNode) serverv.get(serverlist.get(lastChoice));
      return server.serverIP + ":" + server.serverPort;
   }


   /**
    * Method used by the ServiceDiscoveryListener interface to signal the
    * presence of a new server.
    *
    * @param evt  signaling event
    */
   public synchronized void serviceAdded(ServiceDiscoveryEvent evt)
   {
      try
      {
         ServiceItem postItem = evt.getPostEventServiceItem();
         ServerNode server = (ServerNode) postItem.service;
         String servAddr = server.serverIP + ":" + server.serverPort;
         cat.debug("Service Added: ServerNodeAddr=" + servAddr);
         ServiceID serviceID = postItem.serviceID;
         synchronized (addrMap)
         {
            addrMap.put(servAddr, serviceID);
         }
         synchronized (serverv)
         {
            serverv.put(serviceID, server);
         }
         synchronized (serverlist)
         {
            serverlist.add(serviceID);
         }
         cat.debug("Registered services: " + serverv.size());
      }
      catch (Exception err)
      {
         cat.error("Error: " + err.getMessage(), err);
      }
   }


   /**
    * Method used by the ServiceDiscoveryListener interface to signal the
    * leaving of a server.
    * This method does nothing if the server has
    * been already removed from the managed lists.
    *
    * @param evt    signaling event
    */
   public synchronized void serviceRemoved(ServiceDiscoveryEvent evt)
   {
      cat.debug("serviceRemoved");
      try
      {
         ServiceItem preItem = evt.getPreEventServiceItem();
         ServerNode server = (ServerNode) preItem.service;
         String servAddr = server.serverIP + ":" + server.serverPort;
         cat.debug("Service Removed: ServerNodeIP=" + servAddr);
         ServiceID serviceID = preItem.serviceID;
         synchronized (addrMap)
         {
            addrMap.remove(servAddr);
         }
         synchronized (serverv)
         {
            serverv.remove(serviceID);
         }
         synchronized (serverlist)
         {
            serverlist.remove(serviceID);
         }
         cat.debug("Registered services: " + serverv.size());
      }
      catch (Exception err)
      {
         cat.error("Error: " + err.getMessage(), err);
      }
   }


   /**
    * Method invoked by the proxy if it noticed a server is not accessible
    *
    * @param failedAddr  address of the failed server
    */
   public synchronized void serviceFailure(String failedAddr)
   {
      cat.debug("SserviceFailure: failedAddr=" + failedAddr);
      ServiceID serviceID = (ServiceID) addrMap.get(failedAddr);
      // if a server is stopped using shutdown command
      // its serviceID has been removed:
      if(serviceID!=null)
      {
         synchronized (addrMap)
         {
            addrMap.remove(failedAddr);
         }
         synchronized (serverv)
         {
            serverv.remove(serviceID);
         }
         synchronized (serverlist)
         {
            serverlist.remove(serviceID);
         }
      }
      cat.debug("Registered services: " + serverv.size());
   }


   /**
    * Method used by the ServiceDiscoveryListener interface to signal the
    * change of a server.
    *
    * @param evt    signaling event
    */
   public synchronized void serviceChanged(ServiceDiscoveryEvent evt)
   {
      cat.warn("serviceChanged: not handled!");
   }


   /**
    * Clear termination of the manager
    */
   public void terminate()
   {
      clientMgr.terminate();
   }

}
