package com.andreani.xtol.test;

import com.andreani.xtol.util.*;
import com.andreani.xtol.proxy.*;
import com.andreani.xtol.node.*;
import org.apache.log4j.Category;
import org.apache.log4j.BasicConfigurator;
import java.io.IOException;
import net.jini.core.lease.*;
import net.jini.space.JavaSpace;
import net.jini.core.transaction.Transaction;
import java.rmi.Naming;
import java.rmi.RMISecurityManager;
import net.jini.discovery.*;
import net.jini.core.lookup.*;
import net.jini.lookup.entry.*;
import net.jini.core.entry.*;
import net.jini.core.transaction.server.*;
import net.jini.discovery.LookupDiscoveryManager;
import net.jini.lookup.ServiceDiscoveryManager;
import net.jini.lookup.LookupCache;
import net.jini.lookup.ServiceDiscoveryEvent;
import net.jini.lookup.ServiceDiscoveryListener;
import java.util.Vector;
import java.util.Hashtable;
import net.jini.core.transaction.TransactionFactory;

/**
 * @author    Stefano Andreani
 * @version   1.0
 */
public class SnoopSpace implements ServiceDiscoveryListener
{
   /**
    * Log handler (Log4Java)
    */
   protected Category cat;
   private Vector tpSpacev = new Vector(10);
   private String spaceToFind;
   private TxManHandler txManHandler;
   private ServiceDiscoveryManager clientMgr = null;


   /**
    * Connects to the JavaSpaces that contain the SessionEntries and registers
    * the listener for new spaces.
    */
   public SnoopSpace()
   {
      cat = Category.getInstance(this.getClass().getName());
      BasicConfigurator.configure();
      cat.debug("contructor");
      txManHandler = new TxManHandler();
      if (System.getSecurityManager() == null)
      {
         System.setSecurityManager(new RMISecurityManager());
      }

      try
      {
         LookupDiscoveryManager mgr = new LookupDiscoveryManager(
               LookupDiscovery.ALL_GROUPS, null, null);
         clientMgr = new ServiceDiscoveryManager(mgr, null);
         // Register for notification of SessionSpaces:
         Class[] classes = new Class[]{JavaSpace.class};
         ServiceTemplate sessionSpacesTempl = new ServiceTemplate(null, classes, null);
         cat.debug("Creating LookupCache");
         LookupCache spaceCache = clientMgr.createLookupCache(sessionSpacesTempl, null, this);
         synchronized(this)
         {
            try
            {
               wait(10000);
            }
            catch (Exception ex)
            {
               cat.debug(ex.getMessage());
            }
         }
      }
      catch (Exception ex)
      {
         cat.error(ex.getMessage(),ex);
      }

   }

   /**
    * Inserts an entry into the space with an empty hashtable.
    *
    */
   public void insertEmptyEntry(String sessionid, String server)
   {
      Hashtable deb = new Hashtable();
      SessionEntry sess = new SessionEntry(sessionid, server, deb, new Boolean(false));
      long timeToLive = Lease.FOREVER;
      Transaction txn = null;
      try
      {
         try
         {
            TransactionManager myTxMgr = txManHandler.getTxMgr();
            txn = (TransactionFactory.create(myTxMgr, Global.TRANS_TIMEOUT)).transaction;
            for (int i = 0; i < tpSpacev.size(); i++)
            {
               ((JavaSpace) tpSpacev.get(i)).write(sess, txn, timeToLive);
               cat.info("Wrote an entry to a space");
            }
         }
         catch (Exception err)
         {
            cat.error("Error: " + err.getMessage(), err);
            txn.abort();
            return;
         }
         txn.commit();
      }
      catch (Exception err)
      {
         cat.error("Error: " + err.getMessage(), err);
         return;
      }

   }


   /**
    * Takes an entry from the JavaSpace
    *
    * @param sessionid  Id of the entry to take
    * @return           Found entry
    */
   public Entry takeEntry()
   {
      cat.debug("takeEntry()");
      Entry result = null;
      try
      {
         long timeToWait = 10000L;
         //TransactionManager myTxMgr = txManHandler.getTxMgr();
         Transaction txn = null;//(TransactionFactory.create(myTxMgr, Global.TRANS_TIMEOUT)).transaction;
         result = ((JavaSpace) tpSpacev.get(0)).take(null, txn, timeToWait);
         cat.debug("Read an entry from the space");
         cat.debug("and the result was: " + result);
      }
      catch (Exception err)
      {
         cat.error("Error: " + err.getMessage(), err);
      }
      return result;
   }

   public void serviceAdded(ServiceDiscoveryEvent evt)
   {
      cat.debug("serviceAdded");
      try
      {
         ServiceItem postItem = evt.getPostEventServiceItem();
         tpSpacev.add(postItem.service);
         populateSpace(tpSpacev.size() - 1);
      }
      catch (Exception err)
      {
         cat.error("Error: " + err.getMessage(), err);
      }
   }


   /**
    * Description of the Method
    *
    * @param event  Description of Parameter
    */
   public void serviceRemoved(ServiceDiscoveryEvent event)
   {
   }


   /**
    * Description of the Method
    *
    * @param event  Description of Parameter
    */
   public void serviceChanged(ServiceDiscoveryEvent event)
   {
   }


   /**
    * Description of the Method
    *
    * @param entry       Description of Parameter
    */
   public void writeEntry(Entry entry)
   {
      cat.debug("writeEntry()");
      Transaction txn = null;
      try
      {
         try
         {
            //TransactionManager myTxMgr = txManHandler.getTxMgr();
            //txn = (TransactionFactory.create(myTxMgr, Global.TRANS_TIMEOUT)).transaction;
            for (int i = 0; i < tpSpacev.size(); i++)
            {
               JavaSpace tpSpacei = (JavaSpace) tpSpacev.get(i);
               tpSpacei.write(entry, txn, net.jini.core.lease.Lease.FOREVER);
               cat.info("Entry added into a space:");
               cat.info("New Entry:\n"+entry);
            }
         }
         catch (Exception err)
         {
            cat.error("Error: " + err.getMessage(), err);
            txn.abort();
            return;
         }
         txn.commit();
      }
      catch (Exception err)
      {
         cat.error("Error: " + err.getMessage(), err);
         return;
      }
   }

   /**
    * This method populate a new JavaSpace with SessionEntries contained in the
    * other spaces. ServiceDiscoveryListener docs suggests to simply note the
    * occurrence of the ServiceDiscoveryEvent, and perform any time consuming
    * event handling asynchronously, so this method needs to be fast, otherwise
    * it needs to be invoked asynchronously.
    *
    * @param i  Index of the space to populate
    */
   public void populateSpace(int i)
   {
      /*
       * For now I suppose that JavaSpaces are created before TolerantServlets and
       * that they don't fail, so I don't need to polulate a new space.
       */
   }

   public void updateReferences(String oldAddr, String newAddr)
   {
      // This method is very unefficient, but I haven't enough time to change
      // it now ;)
      cat.debug("updateReferences()");
      Transaction txn = null;
      SessionEntry template = new SessionEntry(null, oldAddr, null, new Boolean(false));
      try
      {
         try
         {
            TransactionManager myTxMgr = txManHandler.getTxMgr();
            txn = (TransactionFactory.create(myTxMgr, Global.TRANS_TIMEOUT)).transaction;
            for (int i = 0; i < tpSpacev.size(); i++)
            {
               JavaSpace tpSpacei = (JavaSpace) tpSpacev.get(i);
               SessionEntry oldEntry;
               // I have at last an entry:
               while((oldEntry = (SessionEntry) tpSpacei.takeIfExists(template, txn, 0))!=null)
               {
                  SessionEntry updatedEntry = new SessionEntry(null, newAddr, oldEntry.session,
                     new Boolean(false));
                  tpSpacei.write(updatedEntry, txn, net.jini.core.lease.Lease.FOREVER);
                  cat.info("Entry updated into a space");
               }
            }
         }
         catch (Exception err)
         {
            cat.error("Error: " + err.getMessage(), err);
            txn.abort();
            return;
         }
         txn.commit();
      }
      catch (Exception err)
      {
         cat.error("Error: " + err.getMessage(), err);
         return;
      }
   }


   public void terminate()
   {
      clientMgr.terminate();
   }

   void snoop()
   {
      cat.debug("snoop()");
      Entry[] sev = new Entry[20];
      sev[0] = takeEntry();
      int i = 0;
      for (i=0; sev[i] != null; )
      {
         cat.debug(sev[i]);
         i++;
         sev[i] = takeEntry();
      }
      for (int j=0; j < i; j++)
      {
         writeEntry(sev[j]);
      }
   }

   public static void main(String[] args)
   {
      SnoopSpace ss = new SnoopSpace();
      ss.snoop();
      System.exit(0);
   }
}
