package com.andreani.xtol.util;

import com.andreani.xtol.*;
import net.jini.entry.AbstractEntry;
import java.util.*;

/**
 * This class is the Entry object contained into Session Spaces that stores
 * user state and his bound server address.
 * @author    Stefano Andreani
 * @version   1.0
 */
public class SessionEntry extends AbstractEntry
{
   /**
    * This is the Global Unique Id that identify user session.
    */
   public String id;
   /**
    * The address of destination server
    */
   public String serverAddr;
   /**
    * This Hashtable contains all objects representing user state.
    */
   public Hashtable session;
   /**
    * Fail is true if the server which was servicing has failed and the new one
    * assigned hasn't yet copied this session into its cache.
    */
   public Boolean fail;


   /**
    * Empty Constructor for the SessionEntry object
    */
   public SessionEntry()
   {
   }


   /**
    * Constructor for the SessionEntry object
    *
    * @param id_p          Session GUID
    * @param serverAddr_p  IP and port
    * @param obj           user state
    * @param fail_p        fail flag
    */
   public SessionEntry(String id_p, String serverAddr_p, Hashtable obj, Boolean fail_p)
   {
      id = id_p;
      serverAddr = serverAddr_p;
      session = obj;
      fail = fail_p;
   }


   /**
    * Formatted output, useful for debugging
    *
    * @return   String representation of this object
    */
   public String toString()
   {
      StringBuffer temp = new StringBuffer();
      temp.append("\nSession id=" + id + "#\n" +
            "serverAddr=" + serverAddr + "#\n" +
            "fail flag=" + fail.booleanValue() + "\n" +
            "Session attributes:\n");
      for (Enumeration e = session.keys(); e.hasMoreElements(); )
      {
         Object key = e.nextElement();
         temp.append(key + "=" + session.get(key) + "\n");
      }
      return temp.toString();
   }
}
