package com.andreani.xtol.util;

import com.andreani.xtol.*;
import org.apache.log4j.Category;
import org.apache.log4j.BasicConfigurator;
import java.io.IOException;
import net.jini.core.lease.*;
import net.jini.space.JavaSpace;
import net.jini.core.transaction.Transaction;
import java.rmi.Naming;
import java.rmi.RMISecurityManager;
import net.jini.discovery.*;
import net.jini.core.lookup.*;
import net.jini.lookup.entry.*;
import net.jini.core.entry.*;
import net.jini.core.transaction.server.*;
import net.jini.discovery.LookupDiscoveryManager;
import net.jini.lookup.ServiceDiscoveryManager;
import net.jini.lookup.LookupCache;
import net.jini.lookup.ServiceDiscoveryEvent;
import net.jini.lookup.ServiceDiscoveryListener;
import java.util.Vector;
import java.util.Hashtable;
import net.jini.core.transaction.TransactionFactory;

/**
 * This class manages available JavaSpaces putting and getting entries.
 *
 * @author    Stefano Andreani
 * @version   1.0
 */
public class SessionSpaceManager implements ServiceDiscoveryListener
{
   /**
    * Log handler (Log4Java)
    */
   protected Category cat;
   private Vector tpSpacev = new Vector(10);
   private TxManHandler txManHandler;
   private ServiceDiscoveryManager clientMgr = null;


   /**
    * Connects to the JavaSpaces that contain the SessionEntries and registers
    * the listener for new spaces.
    */
   public SessionSpaceManager()
   {
      cat = Category.getInstance(this.getClass().getName());
      //BasicConfigurator.configure();
      cat.debug("contructor");
      txManHandler = new TxManHandler();
      if (System.getSecurityManager() == null)
      {
         System.setSecurityManager(new RMISecurityManager());
      }

      try
      {
         LookupDiscoveryManager mgr = new LookupDiscoveryManager(
               LookupDiscovery.ALL_GROUPS, null, null);
         clientMgr = new ServiceDiscoveryManager(mgr, null);
         // Register for notification of SessionSpaces:
         Class[] classes = new Class[]{JavaSpace.class};
         ServiceTemplate sessionSpacesTempl = new ServiceTemplate(null, classes, null);
         cat.debug("Creating LookupCache");
         LookupCache spaceCache = clientMgr.createLookupCache(sessionSpacesTempl, null, this);
         synchronized(this)
         {
            try
            {
               //wait(Global.LOOKUP_TIMEOUT);
            }
            catch (Exception ex)
            {
               cat.debug(ex.getMessage());
            }
         }
      }
      catch (Exception ex)
      {
         cat.error(ex.getMessage(),ex);
      }
   }

   /**
    * Inserts an entry into the space with an empty hashtable.
    *
    */
   public void insertEmptyEntry(String sessionid, String server)
   {
      Hashtable deb = new Hashtable();
      SessionEntry sess = new SessionEntry(sessionid, server, deb, new Boolean(false));
      long timeToLive = Lease.FOREVER;
      Transaction txn = null;
      try
      {
         try
         {
            TransactionManager myTxMgr = txManHandler.getTxMgr();
            txn = (TransactionFactory.create(myTxMgr, Global.TRANS_TIMEOUT)).transaction;
            for (int i = 0; i < tpSpacev.size(); i++)
            {
               ((JavaSpace) tpSpacev.get(i)).write(sess, txn, timeToLive);
               cat.info("Wrote an entry to a space");
            }
         }
         catch (Exception err)
         {
            cat.error("Error: " + err.getMessage(), err);
            txn.abort();
            return;
         }
         txn.commit();
      }
      catch (Exception err)
      {
         cat.error("Error: " + err.getMessage(), err);
         return;
      }

   }


   /**
    * Find an entry into the space. The caller must be sure the entry is into
    * the space.
    *
    * @param sessionid  id of the entry to find
    * @return           returns the session entry
    */
   public SessionEntry retrieveEntry(String sessionid)
   {
      SessionEntry result = null;
      try
      {
         SessionEntry template = new SessionEntry(sessionid, null, null, null);
         cat.info("Created a template");
         long timeToWait = 20000L;
         TransactionManager myTxMgr = txManHandler.getTxMgr();
         Transaction txn = (TransactionFactory.create(myTxMgr, Global.TRANS_TIMEOUT)).transaction;
         result = (SessionEntry)
               ((JavaSpace) tpSpacev.get(0)).read(template, null/*txn*/, timeToWait);
         txn.commit();
         cat.info("Read an entry from the space");
         String valueToPrint = result.id;
         cat.info("and the result was: " + valueToPrint);
      }
      catch (Exception err)
      {
         cat.error("Error: " + err.getMessage(), err);
      }
      return result;
   }


    public void serviceAdded(ServiceDiscoveryEvent evt)
   {
      cat.debug("serviceAdded");
      try
      {
         ServiceItem postItem = evt.getPostEventServiceItem();
         tpSpacev.add(postItem.service);
         populateSpace(tpSpacev.size() - 1);
      }
      catch (Exception err)
      {
         cat.error("Error: " + err.getMessage(), err);
      }
   }


   public void serviceRemoved(ServiceDiscoveryEvent event)
   {
   }


   public void serviceChanged(ServiceDiscoveryEvent event)
   {
   }

   public void setFailStatus(String sessionid, boolean fail)
   {
      cat.debug("setFailStatus():"+fail);
      Transaction txn = null;
      SessionEntry template = new SessionEntry(sessionid, null, null, null);
      try
      {
         try
         {
            TransactionManager myTxMgr = txManHandler.getTxMgr();
            txn = (TransactionFactory.create(myTxMgr, Global.TRANS_TIMEOUT)).transaction;
            for (int i = 0; i < tpSpacev.size(); i++)
            {
               JavaSpace tpSpacei = (JavaSpace) tpSpacev.get(i);
               SessionEntry read = (SessionEntry)tpSpacei.take(template, null/*txn*/, 0);
               read.fail=new Boolean(fail);
               tpSpacei.write(read, null/*txn*/, net.jini.core.lease.Lease.FOREVER);
               cat.debug("setFail done.");
            }
         }
         catch (Exception err)
         {
            cat.error("Error: " + err.getMessage(), err);
            txn.abort();
            return;
         }
         txn.commit();
      }
      catch (Exception err)
      {
         cat.error("Error: " + err.getMessage(), err);
         return;
      }
   }

   public void updateSession(SessionEntry entry)
   {
      cat.debug("updateSession()");
      Transaction txn = null;
      SessionEntry template = new SessionEntry(entry.id, null, null, null);
      cat.debug("entry.id="+entry.id);
      try
      {
         try
         {
            TransactionManager myTxMgr = txManHandler.getTxMgr();
            txn = (TransactionFactory.create(myTxMgr, Global.TRANS_TIMEOUT)).transaction;
            for (int i = 0; i < tpSpacev.size(); i++)
            {
               JavaSpace tpSpacei = (JavaSpace) tpSpacev.get(i);
               SessionEntry read = (SessionEntry)tpSpacei.take(template, null/*txn*/, 0);
               cat.debug("read="+read);
               cat.debug("read.fail="+read.fail.booleanValue());
               if(!read.fail.booleanValue())
               {
                  tpSpacei.write(entry, null/*txn*/, net.jini.core.lease.Lease.FOREVER);
                  cat.debug("Entry updated into a space:");
                  cat.debug("Old Entry:\n"+read);
                  cat.debug("New Entry:\n"+entry);
               }
               else
               {
                  // rollback ...
                  tpSpacei.write(read, null/*txn*/, net.jini.core.lease.Lease.FOREVER);
               }
            }
         }
         catch (Exception err)
         {
            cat.error("Error: " + err.getMessage(), err);
            txn.abort();
            return;
         }
         txn.commit();
      }
      catch (Exception err)
      {
         cat.error("Error: " + err.getMessage(), err);
         return;
      }
   }

   public void snoopSpace()
   {
      cat.debug("snoopSpace()");
      Transaction txn = null;
      SessionEntry template = new SessionEntry(null, null, null, null);
      try
      {
         try
         {
            TransactionManager myTxMgr = txManHandler.getTxMgr();
            txn = (TransactionFactory.create(myTxMgr, Global.TRANS_TIMEOUT)).transaction;
            for (int i = 0; i < tpSpacev.size(); i++)
            {
               JavaSpace tpSpacei = (JavaSpace) tpSpacev.get(i);
               SessionEntry read;
               while((read = (SessionEntry)tpSpacei.take(template, txn, 0))!=null)
               {
                  cat.debug("read="+read);
               }
            }
         }
         catch (Exception err)
         {
            cat.error("Error: " + err.getMessage(), err);
            txn.abort();
            return;
         }
         txn.abort();
      }
      catch (Exception err)
      {
         cat.error("Error: " + err.getMessage(), err);
         return;
      }
   }



   /**
    * This method populate a new JavaSpace with SessionEntries contained in the
    * other spaces. ServiceDiscoveryListener docs suggests to simply note the
    * occurrence of the ServiceDiscoveryEvent, and perform any time consuming
    * event handling asynchronously, so this method needs to be fast, otherwise
    * it needs to be invoked asynchronously.
    *
    * @param i  Index of the space to populate
    */
   public void populateSpace(int i)
   {
      /*
       * For now I suppose that JavaSpaces are created before TolerantServlets and
       * that they don't fail, so I don't need to polulate a new space.
       */
   }

   public void updateReferences(String oldAddr, String newAddr)
   {
      // This method is very unefficient, but I haven't enough time to change
      // it now ;)
      cat.debug("updateReferences()");
      Transaction txn = null;
      SessionEntry template = new SessionEntry(null, oldAddr, null, null);
      try
      {
         try
         {
            TransactionManager myTxMgr = txManHandler.getTxMgr();
            txn = (TransactionFactory.create(myTxMgr, Global.TRANS_TIMEOUT)).transaction;
            for (int i = 0; i < tpSpacev.size(); i++)
            {
               JavaSpace tpSpacei = (JavaSpace) tpSpacev.get(i);
               SessionEntry oldEntry;
               // I have at last an entry:
               while((oldEntry = (SessionEntry) tpSpacei.takeIfExists(template, null/*txn*/, 0))!=null)
               {
                  cat.info("Old Entry to update:\n"+oldEntry);
                  SessionEntry updatedEntry = new SessionEntry(oldEntry.id, newAddr,oldEntry.session, new Boolean(true));
                  tpSpacei.write(updatedEntry, null/*txn*/, net.jini.core.lease.Lease.FOREVER);
                  cat.info("Updated Entry:\n"+updatedEntry);
               }
            }
         }
         catch (Exception err)
         {
            cat.error("Error: " + err.getMessage(), err);
            txn.abort();
            return;
         }
         txn.commit();
      }
      catch (Exception err)
      {
         cat.error("Error: " + err.getMessage(), err);
         return;
      }
   }


   public void terminate()
   {
      clientMgr.terminate();
   }
}
