package com.andreani.xtol.util;

import com.andreani.xtol.*;
import net.jini.discovery.LookupDiscoveryManager;
import net.jini.lookup.ServiceDiscoveryManager;
import net.jini.discovery.*;
import net.jini.core.lookup.*;
import net.jini.lookup.ServiceDiscoveryEvent;
import net.jini.lookup.ServiceDiscoveryListener;
import org.apache.log4j.Category;
import org.apache.log4j.BasicConfigurator;
import net.jini.core.transaction.server.*;
import net.jini.core.transaction.Transaction;
import net.jini.lookup.entry.*;
import net.jini.core.entry.*;
import java.rmi.RMISecurityManager;
import java.rmi.RemoteException;

/**
 * Manager class for transactions.
 *
 * @author    Stefano Andreani
 * @version   1.0
 */
public class TxManHandler implements ServiceDiscoveryListener
{
   private TransactionManager txMgr = null;
   protected Category cat;
   private ServiceDiscoveryManager clientMgr = null;

   /**
    * Constructor for the TxManHandler object
    */
   public TxManHandler()
   {
      cat = Category.getInstance(this.getClass().getName());
      cat.debug("contructor");
      if (System.getSecurityManager() == null)
      {
         System.setSecurityManager(new RMISecurityManager());
      }

      try
      {
         cat.debug("new LookupDiscoveryManager");
         LookupDiscoveryManager mgr = new LookupDiscoveryManager(
               LookupDiscovery.ALL_GROUPS, null, null);
         cat.debug("new ServiceDiscoveryManager");
         clientMgr = new ServiceDiscoveryManager(mgr, null);
         // Register for notification of TxManager:
         Entry[] txName = new Entry[]{new Name(Global.TX_MANAGER_NAME)};
         cat.debug("new ServiceTemplate");
         ServiceTemplate txManTempl = new ServiceTemplate(null,null,txName);
         cat.debug("clientMgr.lookup(");
         ServiceItem txItem = clientMgr.lookup(txManTempl, null, Global.LOOKUP_TIMEOUT);
         cat.debug("txItem.service="+txItem.service);
         txMgr = (TransactionManager) txItem.service;
      }
      catch (Exception ex)
      {
         cat.error(ex.getMessage(),ex);
      }
   }


   public void serviceAdded(ServiceDiscoveryEvent evt)
   {
      cat.debug("TxManager found");
      try
      {
         ServiceItem postItem = evt.getPostEventServiceItem();
         txMgr = (TransactionManager) postItem.service;
      }
      catch (Exception err)
      {
         cat.error("Error: " + err.getMessage(), err);
      }
   }


   public void serviceRemoved(ServiceDiscoveryEvent event)
   {
   }


   public void serviceChanged(ServiceDiscoveryEvent event)
   {
   }


   /**
    * Gets the TxMgr attribute of the TxManHandler object
    */
   public TransactionManager getTxMgr()
   {
      if (txMgr != null)
      {
         return txMgr;
      }
      else
      {
         Entry[] txName = new Entry[]{new Name(Global.TX_MANAGER_NAME)};
         ServiceTemplate txManTempl = new ServiceTemplate(null,null,txName);
         try
         {
            ServiceItem txItem = clientMgr.lookup(txManTempl, null, Global.LOOKUP_TIMEOUT);
            txMgr = (TransactionManager) txItem.service;
         }
         catch (Exception ex)
         {
            cat.error(ex.getMessage(),ex);
         }
         return txMgr;
      }
   }
}
